// created on 05/10/2004 at 20:48
using System;


public class XmlBioSeqDocument : System.Xml.XmlDocument {
	System.Xml.XmlNode _all_seq;
	int _id_number;


	public XmlBioSeqDocument () {
		System.Xml.XmlElement Bsml = this.CreateElement ("Bsml");
		this.AppendChild(Bsml);
		System.Xml.XmlElement Definitions = this.CreateElement ("Definitions");
		Bsml.AppendChild(Definitions);
		_all_seq = this.CreateElement ("Sequences");
		Definitions.AppendChild(_all_seq);
		_id_number = 0;
	}
	
	override public void Save (string filename) {
		if (filename.EndsWith(".fasta")) {
			this.SaveFasta(filename);
		}
		else {
			base.Save(filename);
		}
	}

	override public void Load (string filename) {
		try {
			base.Load(filename);
			System.Xml.XmlNodeList liste = this.GetElementsByTagName ("Sequences");
			_all_seq = liste[0];
		}
		catch (System.Xml.XmlException e) {
			System.Xml.XmlElement Bsml = this.CreateElement ("Bsml");
			this.AppendChild(Bsml);
			System.Xml.XmlElement Definitions = this.CreateElement ("Definitions");
			Bsml.AppendChild(Definitions);
			_all_seq = this.CreateElement ("Sequences");
			Definitions.AppendChild(_all_seq);
			//Console.WriteLine("Generic Exception Handler: {0}", e.ToString());
			this.LoadFasta(filename);
		}
	}
	
	public System.Xml.XmlNode AddSequence(System.Xml.XmlNode xmlnode) {
		//copy this node: problem
		System.Xml.XmlNode one_seq = this.ImportNode (xmlnode, true);
		_all_seq.AppendChild(one_seq);
		return (one_seq);
	}
	
	public System.Xml.XmlNode AddSequence(string name, string sequence) {
		//Console.WriteLine("System.Xml.XmlNode AddSequence"+name+sequence);
		System.Xml.XmlElement one_seq = this.CreateElement ("Sequence");
		one_seq.SetAttribute("id", "seq_"+this._id_number);
		if (name == "") name = "seq_"+this._id_number;
		this._id_number++;
		one_seq.SetAttribute("title", name);
		one_seq.SetAttribute("molecule", "dna");
		System.Xml.XmlElement seqdata = this.CreateElement ("Seq-data");
		seqdata.InnerText = sequence;
		one_seq.AppendChild(seqdata);
		_all_seq.AppendChild(one_seq);
		return (one_seq);
	}
	
	public void SaveFasta(string filename) {
		System.IO.StreamWriter fp = new System.IO.StreamWriter(filename);
		string sequence = "";
		int offset;
		
		System.Xml.XmlNodeList liste_seq = this.GetElementsByTagName ("Sequence");
		for (int i=0; i < liste_seq.Count; i++) {
			fp.WriteLine(">"+liste_seq[i].Attributes["title"].Value);
			sequence = liste_seq[i].SelectNodes("Seq-data")[0].InnerText;
			while (sequence.Length > 0) {
				offset = sequence.Length;
				if (offset > 80) offset = 80;
				fp.WriteLine(sequence.Substring(0,offset));
				if (offset == 80) sequence = sequence.Substring(offset);
				else sequence = "";
				
			}
		//	fp.WriteLine(sequence);
		}
		fp.Close();
		
	}
	
	public void LoadFasta(string filename) {
		//AddSequence("", "");
		string line="";
		string sequence="";
		string name=null;
		//this.Save(System.Console.Out);
		System.IO.StreamReader fp = new System.IO.StreamReader(filename);
		while ((line = fp.ReadLine()) != null ) {
		//Console.WriteLine("LoadFasta(string filename)"+line);
			if (line.Length == 0) continue;
			if (line[0] == '>') {
				if (name != null) {
					this.AddSequence(name,sequence);
					sequence = "";
				}
				name = line.Substring(1);
			}
			else {
				sequence += line;
			}
		}
		fp.Close();
		if (name != null) this.AddSequence(name,sequence);
	}
}